@@ -30,6 +30,7 @@
 #include "goaprovider.h"
 #include "goaprovider-priv.h"
 #include "goatelepathyprovider.h"
+#include "goautils.h"
 
 typedef struct _GoaTelepathyProviderPrivate GoaTelepathyProviderPrivate;
 
@@ -1027,40 +1028,13 @@ goa_telepathy_provider_finalize (GObject *object)
   (G_OBJECT_CLASS (goa_telepathy_provider_parent_class)->finalize) (object);
 }
 
-static void
-initialize_client_factory (void)
-{
-  TpSimpleClientFactory *factory;
-  TpAccountManager *account_manager;
-  GQuark account_features[] = {
-      TP_ACCOUNT_FEATURE_STORAGE,
-      TP_ACCOUNT_FEATURE_CONNECTION,
-      0};
-  GQuark connection_features[] = {
-      TP_CONNECTION_FEATURE_AVATAR_REQUIREMENTS,
-      TP_CONNECTION_FEATURE_CONTACT_INFO,
-      0};
-
-  /* We make sure that new instances of Telepathy objects will have all
-   * the features we need. */
-  factory = tp_simple_client_factory_new (NULL);
-  tp_simple_client_factory_add_account_features (factory, account_features);
-  tp_simple_client_factory_add_connection_features (factory, connection_features);
-
-  account_manager = tp_account_manager_new_with_factory (factory);
-  tp_account_manager_set_default (account_manager);
-
-  g_object_unref (account_manager);
-  g_object_unref (factory);
-}
-
 static void
 goa_telepathy_provider_class_init (GoaTelepathyProviderClass *klass)
 {
   GObjectClass *object_class = G_OBJECT_CLASS (klass);
   GoaProviderClass *provider_class = GOA_PROVIDER_CLASS (klass);
 
-  initialize_client_factory ();
+  goa_utils_initialize_client_factory ();
 
   object_class->constructed  = goa_telepathy_provider_constructed;
   object_class->finalize     = goa_telepathy_provider_finalize;
