@@ -359,23 +359,52 @@ out:
 
 #ifdef HAVE_TIMERFD
 static gboolean
-on_timer_source_ready (GObject *stream, GoaAlarm *self)
+on_timer_source_ready (GObject *stream, GTask *task)
 {
   gint64 number_of_fires;
   gssize bytes_read;
   gboolean run_again = FALSE;
+  GError *error = NULL;
+  GoaAlarm *self;
+  GCancellable *cancellable;
+
+  self = g_task_get_source_object (task);
+  cancellable = g_task_get_cancellable (task);
 
   g_return_val_if_fail (GOA_IS_ALARM (self), FALSE);
-  g_return_val_if_fail (self->priv->type == GOA_ALARM_TYPE_TIMER, FALSE);
 
   g_rec_mutex_lock (&self->priv->lock);
-  if (g_cancellable_is_cancelled (self->priv->cancellable))
+
+  if (self->priv->type == GOA_ALARM_TYPE_UNSCHEDULED)
+    {
+      goa_debug ("GoaAlarm: timer source was unscheduled after "
+                 "callback was invoked, but before callback got "
+                 "the lock.");
+      goto out;
+    }
+  else if (self->priv->type != GOA_ALARM_TYPE_TIMER)
+    {
+      goa_warning ("GoaAlarm: timer source ready callback called "
+                   "when timer source isn't supposed to be used. "
+                   "Current timer type is %u", self->priv->type);
+      goto out;
+    }
+
+  if (g_cancellable_is_cancelled (cancellable))
     goto out;
 
   bytes_read =
     g_pollable_input_stream_read_nonblocking (G_POLLABLE_INPUT_STREAM (stream),
                                               &number_of_fires, sizeof (gint64),
-                                              NULL, NULL);
+                                              NULL, &error);
+
+  if (bytes_read < 0)
+    {
+      goa_warning ("GoaAlarm: failed to read from timer fd: %s\n",
+                   error->message);
+      g_error_free (error);
+      goto out;
+    }
 
   if (bytes_read == sizeof (gint64))
     {
@@ -394,9 +423,14 @@ out:
 }
 
 static void
-clear_timer_source_pointer (GoaAlarm *self)
+clear_timer_source (GTask *task)
 {
+  GoaAlarm *self;
+
+  self = g_task_get_source_object (task);
   self->priv->timer.source = NULL;
+
+  g_object_unref (task);
 }
 #endif
 
@@ -408,6 +442,7 @@ schedule_wakeups_with_timerfd (GoaAlarm *self)
   int fd;
   int result;
   GSource *source;
+  GTask *task;
   static gboolean seen_before = FALSE;
 
   if (!seen_before)
@@ -440,14 +475,16 @@ schedule_wakeups_with_timerfd (GoaAlarm *self)
   self->priv->type = GOA_ALARM_TYPE_TIMER;
   self->priv->timer.stream = g_unix_input_stream_new (fd, TRUE);
 
+  task = g_task_new (self, self->priv->cancellable, NULL, NULL);
+
   source =
     g_pollable_input_stream_create_source (G_POLLABLE_INPUT_STREAM
                                            (self->priv->timer.stream),
                                            self->priv->cancellable);
   self->priv->timer.source = source;
   g_source_set_callback (self->priv->timer.source,
-                         (GSourceFunc) on_timer_source_ready, self,
-                         (GDestroyNotify) clear_timer_source_pointer);
+                         (GSourceFunc) on_timer_source_ready, task,
+                         (GDestroyNotify) clear_timer_source);
   g_source_attach (self->priv->timer.source, self->priv->context);
   g_source_unref (source);
 
@@ -572,6 +609,7 @@ goa_alarm_set_time (GoaAlarm *self, GDateTime *time, GCancellable *cancellable)
   if (g_cancellable_is_cancelled (cancellable))
     return;
 
+  g_rec_mutex_lock (&self->priv->lock);
   if (self->priv->cancellable != NULL && self->priv->cancellable != cancellable)
     g_cancellable_cancel (self->priv->cancellable);
 
@@ -601,12 +639,12 @@ goa_alarm_set_time (GoaAlarm *self, GDateTime *time, GCancellable *cancellable)
 
   self->priv->context = g_main_context_ref (g_main_context_default ());
 
-  g_object_notify (G_OBJECT (self), "time");
-
   schedule_wakeups (self);
 
   /* Wake up right away, in case it's already expired leaving the gate */
   schedule_immediate_wakeup (self);
+  g_rec_mutex_unlock (&self->priv->lock);
+  g_object_notify (G_OBJECT (self), "time");
 }
 
 GDateTime *
